<?php
/**
 * SuiteCRM is a customer relationship management program developed by SalesAgility Ltd.
 * Copyright (C) 2021 SalesAgility Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by the
 * Free Software Foundation with the addition of the following permission added
 * to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED WORK
 * IN WHICH THE COPYRIGHT IS OWNED BY SALESAGILITY, SALESAGILITY DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Supercharged by SuiteCRM" logo. If the display of the logos is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Supercharged by SuiteCRM".
 */


namespace App\Data\Service;

use App\Engine\Service\ActionAvailabilityChecker\ActionAvailabilityChecker;
use App\Process\Service\ActionDefinitionProvider;

class RecordActionDefinitionProvider extends ActionDefinitionProvider implements RecordActionDefinitionProviderInterface
{
    /**
     * @var array
     */
    private $recordViewActions;

    /**
     * BulkActionDefinitionProvider constructor.
     * @param array $recordViewActions
     * @param ActionAvailabilityChecker $actionChecker
     */
    public function __construct(array $recordViewActions, ActionAvailabilityChecker $actionChecker)
    {
        parent::__construct($actionChecker);
        $this->recordViewActions = $recordViewActions;
    }

    /**
     * @inheritDoc
     */
    public function getActions(string $module, array $moduleDefaults = []): array
    {
        $config = $this->recordViewActions;
        $config['modules'][$module] = $config['modules'][$module] ?? [];
        $config['modules'][$module]['actions'] = $config['modules'][$module]['actions'] ?? [];
        $config['modules'][$module]['exclude'] = $config['modules'][$module]['exclude'] ?? [];

        $config['modules'][$module]['actions'] = array_merge($moduleDefaults['actions'] ?? [],
            $config['modules'][$module]['actions']);
        $config['modules'][$module]['exclude'] = array_merge($moduleDefaults['exclude'] ?? [],
            $config['modules'][$module]['exclude']);

        return $this->filterActions($module, $config);
    }
}
